from cs_ai_common.logging.internal_logger import InternalLogger
from cs_ai_common.rds.db_connection import RdsDbConnection
from cs_ai_common.rds.entities.entities import Base
from cs_ai_common.rds.entities.entities import Advertisement, Location

def create_db_schema() -> None:
    InternalLogger.LogDebug("Creating database schema")
    InternalLogger.LogDebug(f"Creating tables: {Base.metadata.tables.keys()}")
    rds_db_connection = RdsDbConnection()
    engine = rds_db_connection.get_engine()
    Base.metadata.create_all(engine)


from sqlalchemy.orm import declarative_base
from datetime import datetime
from sqlalchemy import Column, DateTime, ForeignKey, Integer, String
from sqlalchemy.orm import relationship

Base = declarative_base()

class Location(Base):
    __tablename__ = "locations"

    id = Column(Integer(), primary_key=True)
    country = Column(String(50), nullable=False)
    city = Column(String(50), nullable=False)
    postal_code = Column(String(20), nullable=False)
    advertisements = relationship('Advertisement', back_populates='location')

class Advertisement(Base):
    __tablename__ = "advertisements"
    
    id = Column(Integer(), primary_key=True)
    make = Column(String(50), nullable=False)
    model = Column(String(50), nullable=False)
    production_year = Column(Integer(), nullable=False)
    engine_capacity = Column(Integer(), nullable=False)
    engine_power = Column(Integer(), nullable=False)
    mileage = Column(Integer(), nullable=False)
    transmission = Column(String(30), nullable=False)
    fuel_type = Column(String(30), nullable=False)
    cache_hash = Column(String(100), nullable=False)
    source = Column(String(30), nullable=False)
    link = Column(String(200), nullable=False)
    thumbnail = Column(String(200), nullable=False)
    price = Column(Integer(), nullable=False)
    price_currency = Column(String(20), nullable=False)
    location_code = Column(String(20), nullable=False)
    created_at = Column(DateTime(), default=datetime.now)
    location_id = Column(Integer(), ForeignKey('locations.id'))
    location = relationship('Location', back_populates='advertisements')



from cs_ai_common.dynamodb.utils import build_expression_attribute_names, build_expression_attribute_values, build_update_expression
from botocore.exceptions import ClientError
import boto3
import os
from cs_ai_common.logging.internal_logger import InternalLogger

def update_resolver_task(task_id: str, resolver_name: str, **kwargs) -> None:
    dynamodb = boto3.client('dynamodb')
    table_name = os.getenv("RESULTS_TABLE_NAME")
    try:
        _update_expression = build_update_expression(kwargs)
        _expression_attribute_names = build_expression_attribute_names(kwargs)
        _expression_attribute_values = build_expression_attribute_values(kwargs)    
        InternalLogger.LogDebug(f"UpdateItem: {table_name}, {task_id}, {resolver_name}, {_update_expression}, {_expression_attribute_names}, {_expression_attribute_values}")
        dynamodb.update_item(
            TableName=table_name,
            Key={
                'task_id': {
                    'S': task_id
                },
                'resolver': {
                    'S': resolver_name
                }
            },
            UpdateExpression=_update_expression,
            ExpressionAttributeNames=_expression_attribute_names,
            ExpressionAttributeValues=_expression_attribute_values
        )
        InternalLogger.LogDebug("UpdateItem succeeded")
    except ClientError as e:
        InternalLogger.LogError(f"Unable to update item: {e.response['Error']['Message']}")
        raise e
    
def insert_resolver_result_task(task_id: str, resolver_name: str, **kwargs) -> None:
    RESULTS_TABLE_NAME = os.getenv("RESULTS_TABLE_NAME", "")
    dynamodb = boto3.client('dynamodb')
    item = {
        'task_id': {
            'S': task_id
        },
        'resolver': {
            'S': resolver_name
        },
    }

    for key, value in kwargs.items():
        item[key] = {
            'S': value
        }

    dynamodb.put_item(TableName=RESULTS_TABLE_NAME, Item=item)


import os
from cs_ai_common.dynamodb.utils import build_expression_attribute_names, build_expression_attribute_values, build_update_expression
from cs_ai_common.logging.internal_logger import InternalLogger
import boto3
from botocore.exceptions import ClientError


def update_task_status_failed(task_id: str) -> None:
    update_task(task_id, status="FAILED")
    
def update_task(task_id: str, **kwargs) -> None:
    dynamodb = boto3.client('dynamodb')
    table_name = os.getenv("TASKS_TABLE_NAME")
    try:
        dynamodb.update_item(
            TableName=table_name,
            Key={
                'task_id': {
                    'S': task_id
                }
            },
            UpdateExpression=build_update_expression(kwargs),
            ExpressionAttributeNames=build_expression_attribute_names(kwargs),
            ExpressionAttributeValues=build_expression_attribute_values(kwargs)
        )
        InternalLogger.LogDebug("UpdateItem succeeded")
    except ClientError as e:
        InternalLogger.LogError(f"Unable to update item: {e.response['Error']['Message']}")
        raise e




import logging
import os

LOG_LEVEL = os.getenv("LOG_LEVEL", "INFO")
logger = logging.getLogger(__name__)
logger.setLevel(LOG_LEVEL)
logger.propagate = True


class InternalLogger:
    @staticmethod
    def LogInfo(msg: str):
        logger.info(msg)

    @staticmethod
    def LogDebug(msg: str):
        logger.debug(msg)

    @staticmethod
    def LogError(msg: str):
        logger.error(msg)


import os
from cs_ai_common.resolver_cache.cache_key import build_cache_key
from cs_ai_common.resolver_cache.s3_cache_utils import try_get_cached_result
from cs_ai_common.s3.utils import get_bucket_name_from_arn

def try_get_cache(seed_data: dict, resolver_name: str) -> tuple:
    make, model, production_year = seed_data["Make"], seed_data["Model"], seed_data["ProductionYear"]
    cache_key = build_cache_key(make, model, production_year)
    bucket_name = get_bucket_name_from_arn(os.getenv("CACHE_BUCKET_NAME", None))
    return cache_key, try_get_cached_result(cache_key, resolver_name, bucket_name)